from typing import Literal, Optional
from sqlmodel import Field, SQLModel

class MediaAttachment(SQLModel, table=True):
    id: int = Field(primary_key=True)
    blurhash: str
    description: str
    meta: dict
    preview_url: str
    remote_url: str
    type: Literal['unknown', 'image', 'gifv', 'video', 'audio']
    url: str



from diyalgo.models.emojis import CustomEmoji

from diyalgo.models.attachment import MediaAttachment
from diyalgo.models.tag import Tag
from diyalgo.models.poll import Poll

from diyalgo.models.account import Account, AccountField
from diyalgo.models.status import Status


from typing import Optional, Literal, List, TYPE_CHECKING
from datetime import datetime
from sqlmodel import Field, SQLModel
from bs4 import BeautifulSoup

if TYPE_CHECKING:
    from diyalgo.models import Account, MediaAttachment, Tag, CustomEmoji, Poll


class Mention(SQLModel, table=True):
    acct: str
    id: int
    url: str
    username: str

class Status(SQLModel, table=True):
    """
    Model of a toot on mastodon

    See: https://mastodonpy.readthedocs.io/en/stable/#toot-dicts
    """
    id: int = Field(primary_key=True)
    application: Optional[dict] = None

    account: 'Account'
    bookmarked: Optional[bool] = None
    content: str
    created_at: datetime
    edited_at: Optional[datetime] = None
    emojis: List['CustomEmoji'] = Field(default_factory=list)
    favourited: Optional[bool] = None
    favourites_count: int
    filtered: Optional[List[str]] = Field(default_factory=list)
    in_reply_to_id: int
    in_reply_to_account_id: int
    language: Optional[str] = None
    media_attachments: List['MediaAttachment'] = Field(default_factory=list)
    mentions: List[Mention] = Field(default_factory=list)
    muted: Optional[bool] = None
    pinned: Optional[bool] = None
    poll: Optional['Poll'] = None
    reblog: bool
    reblogged: Optional[bool] = None
    reblogs_count: int
    replies_count: int
    sensitive: bool
    spoiler_text: str
    tags: List['Tag'] = Field(default_factory=list)
    text: Optional[str] = None
    uri: str
    url: str
    visibility: Literal['public', 'unlisted', 'private', 'direct']
    in_reply_to_id: Optional[int] = None
    in_reply_to_account_id: Optional[int] = None

    @property
    def soup(self) -> BeautifulSoup:
        return BeautifulSoup(self.content)


    class Config:
        extra='ignore'




from datetime import datetime
from typing import Optional, List, TYPE_CHECKING
from sqlmodel import Field, SQLModel

if TYPE_CHECKING:
    from diyalgo.models import CustomEmoji

class PollOption(SQLModel):
    title: str
    votes_count: Optional[int] = None

class Poll(SQLModel, table=True):
    id: int = Field(primary_key=True)
    emojis: List['CustomEmoji'] = Field(default_factory=list)
    expires_at: Optional[datetime] = None
    expired: bool
    multiple: bool
    options: List[PollOption] = Field(default_factory=list)
    own_votes: List[int] = Field(default_factory=list)
    voted: Optional[bool] = None
    votes_count: int
    voters_count: Optional[int] = None



from pydantic import BaseModel, AnyHttpUrl
from sqlmodel import Field, SQLModel
from datetime import datetime
from typing import Optional, List
from bs4 import BeautifulSoup

from diyalgo.models import CustomEmoji

class AccountField(BaseModel):
    name: str
    value: str
    verified_at: Optional[datetime] = None
    url: Optional[AnyHttpUrl] = None

    def __init__(self, name:str, value:str, verified_at:Optional[datetime] = None):
        soup = BeautifulSoup(value, 'lxml')
        a = soup.find('a')
        if a is not None:
            url = a.get('href')
        else:
            url = None
        super().__init__(name=name, value=value, url=url, verified_at=verified_at)

    class Config:
        extra = "ignore"


class Account(SQLModel, table=True):
    """https://docs.joinmastodon.org/entities/Account/"""
    id: int = Field(primary_key=True)
    acct: str
    avatar: str
    avatar_static: str
    bot: bool
    created_at:datetime
    discoverable:bool
    display_name:str
    emojis: List[CustomEmoji] = Field(default_factory=list)
    fields: List[AccountField] = Field(default_factory=list)
    followers_count:int
    following_count:int
    group: bool
    header: str
    last_status_at: Optional[datetime] = None
    limited: Optional[bool] = None
    locked: bool
    moved: Optional['Account'] = None
    noindex: Optional[bool] = None
    header_static: str
    note: str
    statuses_count: int
    suspended: Optional[bool] = None
    url: AnyHttpUrl
    username: str

    class Config:
        extra = 'ignore'


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['diyalgo',
 'diyalgo.client',
 'diyalgo.expansions',
 'diyalgo.features',
 'diyalgo.models']

package_data = \
{'': ['*']}

install_requires = \
['Mastodon.py>=1.8.0,<2.0.0',
 'beautifulsoup4>=4.11.1,<5.0.0',
 'lxml>=4.9.2,<5.0.0',
 'pydantic>=1.10.4,<2.0.0',
 'sqlmodel>=0.0.8,<0.0.9']

setup_kwargs = {
    'name': 'diyalgo',
    'version': '0.1.0',
    'description': 'DIY Algorithms for Mastodon',
    'long_description': '# diyalgo\n\nDIY Algoritms for mastodon\n\nJust uploading to PyPI for now to squat on the package name. Will release\nah um a version of this soon :)',
    'author': 'sneakers-the-rat',
    'author_email': 'JLSaunders987@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from sqlmodel import Field, SQLModel

class CustomEmoji(SQLModel, table=True):
    shortcode: str = Field(primary_key=True)
    url: str
    static_url: str
    visible_in_picker: bool
    category: str